@@ -10,9 +10,9 @@ import pandas as pd
 from collections import OrderedDict
 import numbers
 from pyspark.sql import DataFrame
-#from functools import reduce
 
-def listcode_lookup(df, luvar, kodeliste, nokkelverdi):
+
+def listcode_lookup(df, luvar, kodeliste, nokkelverdi, spark_session=None):
     '''
     
     This function adds a new variable to a given dataframe. The added variable contains values 
@@ -26,11 +26,14 @@ def listcode_lookup(df, luvar, kodeliste, nokkelverdi):
     :param nokkelverdi: Python list ([key value, lookup value]). Key value is the name of the 
                         variable in the list of codes which corresponds to the variable of 
                         interest in the dataframe. Lookup value is the name of the variable
-                        in the list of codes to be returned and added to the dataframe.                  
+                        in the list of codes to be returned and added to the dataframe. 
+    :param spark_session: defines the Spark session to be used by the function. Default is None, which
+                        means the function tries to catch the current session.
     :type df: Spark dataframe
     :type luvar: string  
     :type kodeliste: dataframe
     :type nokkelverdi: list 
+    :type spark_session: string
     
     Returns: 
     Dataframe: Spark dataframe consisting of the original dataframe updated with a new variable 
@@ -43,6 +46,11 @@ def listcode_lookup(df, luvar, kodeliste, nokkelverdi):
     if (isinstance(df, DataFrame)) & (isinstance(luvar, str)) & (isinstance(kodeliste, DataFrame)) & (isinstance(nokkelverdi, type([]))):
     
         #Inititerer variabler
+        if spark_session is None:
+            spark = SparkSession.builder.getOrCreate()            
+        else:
+            spark = spark_session
+            
         kodeliste_dict = {}
 
         #Henter nøkkelvariabel og oppslagsvariabel og lager en dictionary av det 
@@ -51,7 +59,7 @@ def listcode_lookup(df, luvar, kodeliste, nokkelverdi):
 
         #Gjør oppslag mot dictionary på variabel vi ønsker og oppretter en egen variabel for resultatet av oppslaget
         mapping_expr = F.create_map([F.lit(x) for x in chain(*kodeliste_dict.items())])
-        df = df.withColumn("{}_kodelisteverdi".format(luvar), mapping_expr.getItem(F.col(luvar))) 
+        df = df.withColumn(nokkelverdi[1].format(luvar), mapping_expr.getItem(F.col(luvar))) 
 
         #Returnere datasettet med ny variabel som resultat av oppslag
         return df
@@ -74,7 +82,7 @@ def listcode_lookup(df, luvar, kodeliste, nokkelverdi):
                 return
             
             
-def missing_correction_bool(df, correction_value=False, exception_for=[], df_name=''):
+def missing_correction_bool(df, correction_value=False, exception_for=[], df_name='', spark_session=None):
     '''
     
     This function checks a dataframe for missing values on Boolean variables and replaces missing values with the 
@@ -85,9 +93,12 @@ def missing_correction_bool(df, correction_value=False, exception_for=[], df_nam
     :param df: The dataframe to be corrected
     :param correction_value: value to replace missing
     :param exception_for: list of variables not to be corrected 
+    :param spark_session: defines the Spark session to be used by the function. Default is None, which
+                        means the function tries to catch the current session.
     :type df: Spark dataframe
     :type correction_value: Boolean 
     :type exception_for: list
+    :type spark_session: string
     
     Returns: corrected dataframe and log in that order
     Dataframe: corrected dataframe (Spark) 
@@ -98,6 +109,10 @@ def missing_correction_bool(df, correction_value=False, exception_for=[], df_nam
     
     if (isinstance(df, DataFrame)) & (isinstance(correction_value, bool)) & (isinstance(exception_for, type([]))):
         #initialiserer variabler
+        if spark_session is None:
+            spark = SparkSession.builder.getOrCreate()            
+        else:
+            spark = spark_session
         boollist = []
 
         #Legger alle boolske variabler i en egen liste
@@ -150,7 +165,7 @@ def missing_correction_bool(df, correction_value=False, exception_for=[], df_nam
             return
 
                                     
-def missing_correction_number(df, correction_value=0, exception_for=[], df_name=''):
+def missing_correction_number(df, correction_value=0, exception_for=[], df_name='', spark_session=None):
     '''
     This function checks a dataframe for missing values on numeric variables and replaces missing values with the 
     correction_value parameter. Correction_value defaults to zero if no value is given.
@@ -160,9 +175,12 @@ def missing_correction_number(df, correction_value=0, exception_for=[], df_name=
     :param df: The dataframe to be corrected
     :param correction_value: value to replace missing
     :param exception_for: list of variables not to be corrected 
+    :param spark_session: defines the Spark session to be used by the function. Default is None, which
+                        means the function tries to catch the current session.               
     :type df: Spark dataframe
     :type correction_value: numeric 
     :type exception_for: list
+    :type spark_session: string
     
     Returns: corrected dataframe and log in that order
     Dataframe: corrected dataframe (Spark) 
@@ -174,6 +192,10 @@ def missing_correction_number(df, correction_value=0, exception_for=[], df_name=
 
     if (isinstance(df, DataFrame)) & (isinstance(correction_value, numbers.Number)) & (isinstance(exception_for, type([]))):
         #initialiserer variabler
+        if spark_session is None:
+            spark = SparkSession.builder.getOrCreate()            
+        else:
+            spark = spark_session
         numlist = []
 
         #Legger alle numeriske variabler i en egen liste
@@ -189,23 +211,23 @@ def missing_correction_number(df, correction_value=0, exception_for=[], df_name=
         
         df_log = []
         if len(df_name)>0:
-            corrected_missing_log = pd.DataFrame(columns=['dataframe', 'datatype', 'variabel', 'korrigeringer'])
+            corrected_missing_log = pd.DataFrame(columns=['dataframe', 'datatype', 'variable', 'corrections'])
             for k, v in df_count.items():
                 if v != 0:
                     df_dict_count= {}
                     df_dict_count['dataframe'] = df_name
-                    df_dict_count['variabel'] = k
+                    df_dict_count['variable'] = k
                     df_dict_count['datatype'] = 'numeric'
-                    df_dict_count['korrigeringer'] = v
+                    df_dict_count['corrections'] = v
                     df_log.append(df_dict_count)
         else:
-            corrected_missing_log = pd.DataFrame(columns=['datatype', 'variabel', 'korrigeringer'])
+            corrected_missing_log = pd.DataFrame(columns=['datatype', 'variable', 'corrections'])
             for k, v in df_count.items():
                 if v != 0:
                     df_dict_count= {}
-                    df_dict_count['variabel'] = k
+                    df_dict_count['variable'] = k
                     df_dict_count['datatype'] = 'numeric'
-                    df_dict_count['korrigeringer'] = v
+                    df_dict_count['corrections'] = v
                     df_log.append(df_dict_count)
 
         df_log_df = pd.DataFrame(df_log)
@@ -227,11 +249,11 @@ def missing_correction_number(df, correction_value=0, exception_for=[], df_name=
             raise Exception('Parameter exception_for må være liste format')
             return
 
-def spark_missing_correction_bool(df, correction_value=False, exception_for=[], df_name=''):
+def spark_missing_correction_bool(df, correction_value=False, exception_for=[], df_name='', spark_session=None):
     #Parameter df --> datasett som det skal kjøres opptelling av missing for
     #Paramater correction_value --> hvilken verdi som skal settes inn istedenfor missing
     #Parameter exception_for --> liste over variable som ikke skal korrigeres
-  
+   
     '''
     This function checks a dataframe for missing values on Boolean variables and replaces missing values with the 
     correction_value parameter. Correction_value defaults to False if no value is given.
@@ -241,9 +263,12 @@ def spark_missing_correction_bool(df, correction_value=False, exception_for=[],
     :param df: The dataframe to be corrected
     :param correction_value: value to replace missing
     :param exception_for: list of variables not to be corrected 
+    :param spark_session: defines the Spark session to be used by the function. Default is None, which
+                        means the function tries to catch the current session.
     :type df: Spark dataframe
     :type correction_value: Boolean 
     :type exception_for: list
+    :type spark_session: string
     
     Returns: corrected dataframe and log in that order
     Dataframe: corrected dataframe (Spark) 
@@ -253,8 +278,11 @@ def spark_missing_correction_bool(df, correction_value=False, exception_for=[],
     '''  
     if (isinstance(df, DataFrame)) & (isinstance(correction_value, bool)) & (isinstance(exception_for, type([]))):
         #initialiserer variabler
-        sc = SparkContext.getOrCreate()
-        sqlContext = SQLContext(sc)
+        if spark_session is None:
+            spark = SparkSession.builder.getOrCreate()            
+        else:
+            spark = spark_session
+            
         boollist = []
 
         #Legger alle boolske variabler i en egen liste
@@ -269,9 +297,9 @@ def spark_missing_correction_bool(df, correction_value=False, exception_for=[],
         
         if len(df_name)>0:
             missing_variabler = [StructField('dataframe', StringType(), False),\
-                   StructField('variabel', StringType(), True),\
+                   StructField('variable', StringType(), True),\
                    StructField('datatype', StringType(), True),\
-                   StructField('korrigeringer', IntegerType(), False)]
+                   StructField('corrections', IntegerType(), False)]
             missing_schema = StructType(missing_variabler)
             
 
@@ -280,29 +308,30 @@ def spark_missing_correction_bool(df, correction_value=False, exception_for=[],
                     if row[k]!=0:
                         df_dict_count= {}
                         df_dict_count['dataframe'] = df_name
-                        df_dict_count['variabel'] = k
+                        df_dict_count['variable'] = k
                         df_dict_count['datatype'] = 'boolean'
-                        df_dict_count['korrigeringer'] = row[k]
+                        df_dict_count['corrections'] = row[k]
                         df_log.append(df_dict_count)
         else:
-            missing_variabler = [StructField('variabel', StringType(), True),\
-                   StructField('korrigeringer', IntegerType(), False)]
+            missing_variabler = [StructField('variable', StringType(), True),\
+                   StructField('datatype', StringType(), True),\
+                   StructField('corrections', IntegerType(), False)]
             missing_schema = StructType(missing_variabler)
             
             for row in df_count.rdd.collect():
                 for k in df_columns:
                     if row[k]!=0:
                         df_dict_count= {}
-                        df_dict_count['variabel'] = k
+                        df_dict_count['variable'] = k
                         df_dict_count['datatype'] = 'boolean'
-                        df_dict_count['korrigeringer'] = row[k]
+                        df_dict_count['corrections'] = row[k]
                         df_log.append(df_dict_count)
         
         if len(df_log)>0:
-            rdd_missing = sc.parallelize(df_log)
-            df_corrections = sqlContext.createDataFrame(rdd_missing, missing_schema)
+            rdd_missing = spark.sparkContext.parallelize(df_log)
+            df_corrections = spark.createDataFrame(rdd_missing, missing_schema)
         else:
-            df_corrections = sqlContext.createDataFrame(sc.emptyRDD(), missing_schema)
+            df_corrections = spark.createDataFrame(spark.sparkContext.emptyRDD(), missing_schema)
             
         #Korrigerer verdier som er boolske
         df = df.fillna(correction_value, subset=boollist)
@@ -321,7 +350,7 @@ def spark_missing_correction_bool(df, correction_value=False, exception_for=[],
             raise Exception('Parameter exception_for må være liste format')
             return
         
-def spark_missing_correction_number(df, correction_value=0, exception_for=[], df_name=''):
+def spark_missing_correction_number(df, correction_value=0, exception_for=[], df_name='', spark_session=None):
     '''
     
     This function checks a dataframe for missing values on numeric variables and replaces missing values with the 
@@ -332,9 +361,12 @@ def spark_missing_correction_number(df, correction_value=0, exception_for=[], df
     :param df: The dataframe to be corrected
     :param correction_value: value to replace missing
     :param exception_for: list of variables not to be corrected 
+    :param spark_session: defines the Spark session to be used by the function. Default is None, which
+                means the function tries to catch the current session.
     :type df: Spark dataframe
     :type correction_value: numeric 
     :type exception_for: list
+    :type spark_session: string
     
     Returns: corrected dataframe and log in that order
     Dataframe: corrected dataframe (Spark) 
@@ -344,8 +376,10 @@ def spark_missing_correction_number(df, correction_value=0, exception_for=[], df
     
     if (isinstance(df, DataFrame)) & (isinstance(correction_value, numbers.Number)) & (isinstance(exception_for, type([]))):
         #initialiserer variabler
-        sc = SparkContext.getOrCreate()
-        sqlContext = SQLContext(sc)
+        if spark_session is None:
+            spark = SparkSession.builder.getOrCreate()            
+        else:
+            spark = spark_session
         numlist = []
 
         #Legger alle numeriske variabler i en egen liste
@@ -362,9 +396,9 @@ def spark_missing_correction_number(df, correction_value=0, exception_for=[], df
         
         if len(df_name)>0:
             missing_variabler = [StructField('dataframe', StringType(), False),\
-                   StructField('variabel', StringType(), True),\
+                   StructField('variable', StringType(), True),\
                    StructField('datatype', StringType(), True),\
-                   StructField('korrigeringer', IntegerType(), False)]
+                   StructField('corrections', IntegerType(), False)]
             missing_schema = StructType(missing_variabler)
             
 
@@ -373,29 +407,30 @@ def spark_missing_correction_number(df, correction_value=0, exception_for=[], df
                     if row[k]!=0:
                         df_dict_count= {}
                         df_dict_count['dataframe'] = df_name
-                        df_dict_count['variabel'] = k
+                        df_dict_count['variable'] = k
                         df_dict_count['datatype'] = 'numeric'
-                        df_dict_count['korrigeringer'] = row[k]
+                        df_dict_count['corrections'] = row[k]
                         df_log.append(df_dict_count)
         else:
             missing_variabler = [StructField('variabel', StringType(), True),\
-                   StructField('korrigeringer', IntegerType(), False)]
+                   StructField('datatype', StringType(), True),\
+                   StructField('corrections', IntegerType(), False)]
             missing_schema = StructType(missing_variabler)
             
             for row in df_count.rdd.collect():
                 for k in df_columns:
                     if row[k]!=0:
                         df_dict_count= {}
-                        df_dict_count['variabel'] = k
+                        df_dict_count['variable'] = k
                         df_dict_count['datatype'] = 'numeric'
-                        df_dict_count['korrigeringer'] = row[k]
+                        df_dict_count['corrections'] = row[k]
                         df_log.append(df_dict_count)
         
         if len(df_log)>0:
-            rdd_missing = sc.parallelize(df_log)
-            df_corrections = sqlContext.createDataFrame(rdd_missing, missing_schema)
+            rdd_missing = spark.sparkContext.parallelize(df_log)
+            df_corrections = spark.createDataFrame(rdd_missing, missing_schema)
         else:
-            df_corrections = sqlContext.createDataFrame(sc.emptyRDD(), missing_schema)
+            df_corrections = spark.createDataFrame(spark.sparkContext.emptyRDD(), missing_schema)
         
              
         #Korrigerer verdier som er numeriske
